var randFloor = require('Utils').randFloor;
var aniState = require('Const').AniNormal;
var enemyType = require('Const').EnemyType;
var voxId = require('Const').VoxId;

cc.Class({
    extends: require('Actor'),

    properties: {
        body: {
            default: [],
            type: [cc.Node]
        },
        subAni: {
            default: [],
            type: [cc.AnimationClip]
        },
        path: {
            default: [],
            type: [cc.AnimationClip]
        },
        velocity: {
            default: cc.Vec2.ONE,
            visible: false
        },
        tempV: {
            default: cc.Vec2.ONE,
            visible: false
        },
        speed: {
            default: 1,
            visible: false
        },
        spt: {
            default: 0,
            visible: false
        },
        target: {
            default: 0,
            visible: false
        },
        init: {
            default: 0,
            visible: false,
            tooltip: 'for decide target'
        },
        initDir: {
            default: 0,
            visible: false
        },
        isInit: {
            default: 0,
            visible: false,
            tooltip: 'for detect move into fish course'
        },
        dirX: {
            default: 0,
            visible: false
        },
        deadCounter: {
            default: 0,
            visible: false
        },
        curDest: {
            default: -1,
            visible: false
        },
        animState: {
            default: null,
            visible: false,
        },
        randPath: {
            default: -1,
            visible: false,
        },
        curPath: {
            default: "",
            visible: false,
        },
        innerName: {
            default: "",
            visible: false,
        },
        owner: {
            default: 0,
            visible: false
        },
        debugCounter: {
            default: 0,
            visible: false,
        },
    },

    onLoad () {
        this._super();
        this._animCtrl = this.node.getComponent(cc.Animation);

        if(this.body != null){
            if(this.body.length > 0){
                this._animBody = this.body[0].getComponent(cc.Animation);
            }
        }

        this._colCtrl = this.node.getComponent(cc.PolygonCollider);
        
        // decide random path
        let randInt = randFloor(0, (10+G.spawn.randPathes.length));
        let self = this;

        if(this.path.length > 0){
            this.randPath = 1;
            randInt = randFloor(0, (this.path.length-1));
            this.curPath = this.path[randInt].name;
            //cc.log("onFishPath, " + this.path[randInt].name);
        }
        else{
            if(randInt >= 10){
                this.randPath = 1;
                this.curPath = G.spawn.randPathes[randInt-10].name;
                
                if(G.spawn.randPathes[randInt-10] == null){
                    cc.log("onRandPath, " + randInt);
                }
            }
        }

        if(this.randPath > 0){
            cc.loader.loadRes("animations/path/common/" + this.curPath, function (err, clip) {
                if(self._animCtrl !== null){
                    self._animCtrl.addClip(clip, self.curPath);
                    self.animState = self._animCtrl.play(self.curPath);
					self.init = 1;
                }
            });
        }
    },

    update (dt) {
        if(this.hp <= 0 && this.body.length <= 0){
            if(this.deadCounter >= 10){
                this.node.rotation = randFloor(1, 359);
                this.deadCounter = 0;
            }

            this.deadCounter++;
            return;
        }

        // if node have animation, just play
        if(this._animCtrl !== null){
            if(this.init == 1){
                //cc.log("What can we play," + this.path.length);

                //for(var i=0; i<this.path.length; i++){
                //    cc.log("animations, " + this.path[i].name);
                //}
                
                //this.curPath = randFloor(0, (this.path.length-1));
                //var rand = 2;
                //cc.log("animations, " + this.path[this.curPath].name);
                //this.animState = this._animCtrl.play(this.path[this.curPath].name);

                // random speed for boss
                if(this.animState != null && this.type >= enemyType.boss_1){
                    this.animState.speed = randFloor(1, 5);
                }

                this.dirX = this.node.position.x;
                this.init = 2;
            }
            else if(this.init == 2){
                //cc.log("node is active, " + this.node.name);
                
                // WTF, boss fish (or big fish) will flash a frame!!
                let bodies = this.node.children;
                bodies[0].active = true;
                this.init = 3;
            }
        }

        if(this.randPath < 0){
            // set tracer properties
            if(this.init == 1){
                let tracer = this.getComponent('Tracer');
                //var rand = randFloor(0, 15);

                // for manual test
                //var rand = 10;
                
                if(this.curDest < 0){
                    this.curDest = randFloor(0, 15);;
                }
                

                let eigen_3 = this.curDest - 3;
                let eigen_7 = this.curDest - 7;
                let eigen_11 = this.curDest - 11;
                let eigen_15 = this.curDest - 15;

                if (tracer.target == null){
                    if(eigen_3 <= 0 && eigen_3 >= -3 && this.spt >= 0 && this.spt <= 3){
                       this.curDest =  randFloor(4, 7);
                    }

                    if(eigen_7 <= 0 && eigen_7 >= -3 && this.spt >= 4 && this.spt <= 7){
                       this.curDest =  randFloor(8, 11);
                    }

                    if(eigen_11 <= 0 && eigen_11 >= -3 && this.spt >= 8 && this.spt <= 11){
                       this.curDest =  randFloor(12, 15);
                    }

                    if(eigen_15 <= 0 && eigen_15 >= -3 && this.spt >= 12 && this.spt <= 15){
                       this.curDest =  randFloor(0, 3);
                    }

                    tracer.target = G.game.tpGroups[this.curDest];
                }

                this.velocity = cc.v2((tracer.target.position.x-this.node.position.x), (tracer.target.position.y-this.node.position.y)).normalizeSelf();
                this.speed = randFloor(100, 200);

                this.dirX = this.node.position.x;

                //cc.log("fish init = " + this.spt + ", " + rand);
                this.init = 0;
            }
            else{
                //cc.log("fish update = " + this.speed);
                this.tempV = this.velocity.clone();
                this.tempV.mulSelf(this.speed);
                this.node.position = this.node.position.addSelf(this.tempV.mul(dt));
            }
        }
    },

    lateUpdate () {
        if(this.hp <= 0){
            if(this._colCtrl !== null){
                this.node.group = "dead fish";
            }

            return;
        }

        this.detectBoundary();
    },

    onEnable () {
        // todo: use counter to limit fish type and generate rate.
        this.schedule(this.addDebugCounter, 1);
    },

    onDisable () {
        this.unschedule(this.addDebugCounter);

        if(G.game.fishCount_com > 0){
            G.game.fishCount_com--;
        }

        G.game.updateHitFishCountText();
    },

    addDebugCounter () {
        this.debugCounter++;
        if(this.debugCounter > 180){
            cc.log("Detected Error Fish, " + this.name + ", " + this.curPath);
            G.game.wtfCounter++;
            this.debugCounter = 0;
        }
    },

    detectBoundary () {
        // check position difference and change fish direction
        if(this.initDir > 5){
            let tempPosX = this.node.position.x;

            //cc.log("fish reverse 0, " + tempPosX + ", " + this.dirX);

            if(tempPosX > this.dirX){
                if(this.node.scaleX > 0){
                    this.node.scaleX = (-1) * this.node.scaleX;
                }
            }
            else if(this.node.scaleX < 0){
                this.node.scaleX = (-1) * this.node.scaleX;
            }

            if(this.dirX != tempPosX){
                this.dirX = tempPosX;
            }

            this.initDir = 0;
        }

        this.initDir++;
    },

    onCollisionEnter (other) {
        if (other.node.group === 'player bullet') {
            this.hp--;
            //this.hp -= 5;

            if(this.type >= enemyType.boss_1){
                // BOSS special rule
                if(G.game.hitSeqFish < 50 && this.hp <= 0){
                    this.hp = 1;
                }
            }

            if(this.hp <= 0){
                //this.node.destroy();
                if(this.hp > -10){
                    this.hp = -11;
                    this.owner = (other.node.name).substring((other.node.name.length-2), (other.node.name.length-1));
                    
                    if(this.body.length > 0){
                        if(this.type >= enemyType.boss_1){
                            this.schedule(this.onDead, 1);
                        }
                        else{
                            // play "die" animation if data exist.
                            this.playSubAnimation(aniState.Die);
                        }

                        if(this.type >= enemyType.boss_1){
                            randInt = 100;
                        }
                    }
                    else{
                        this.schedule(this.onDead, 1);
                    }
                }

                if(this._animCtrl !== null){
                    this._animCtrl.stop(this.curPath);
                }
            }
            else if(this.body.length > 0){
                // play "be hit" animation if data exist.
                this.playSubAnimation(aniState.BeHit);

                // full speed to run away, when HP is low, only for boss.
                if(this.hp <= 5 && this.animState.speed != 10 && this.type >= enemyType.boss_1){
                    this.animState.speed = 10;
/*
                    if(this.node.name == "Fish_poseidon"){
                        // BOSS special rule
                        if(G.game.hitSeqFish < 50){
                            this.hp = 1000;
                        }
                    }
*/
                }
            }
        }

        if (other.node.group === 'inner bound') {
            if(this.isInit == 0){
                if(this.innerName == ""){
                    this.innerName = other.node.name;
                    G.game.fishCount_com++;
                    this.isInit = 1;
                }
            }
            else if(this.innerName != other.node.name){
                this.isInit = 2;
            }
        }

        if (other.node.group === 'outer bound') {
            if(this.isInit == 2){
                if(this.type >= enemyType.boss_1){
                    G.game.bossCount--;
                    G.sound.setBossTimer();
                }
                //else{
                //    G.game.fishCount_com--;
                //}
                    
                this.node.destroy();
                
                // todo: more smart GC    
                //cc.sys.garbageCollect();
                //cc.log("onOutofBound, " + G.game.fishCount);
            }
        }
    },

    playSubAnimation (index) {
        //cc.log("playSubAnimation, " + this.subAni[index].name);

        if(index == aniState.Die){
            this._animBody.stop();
        }

        this._animBody.playAdditive(this.subAni[index].name);
    },

    checkAudio(list, index){
        switch(list[index].type){
            case audioType.Vox_m_fish:
                return list[index].m_fish;
                break;
            case audioType.Vox_m_fish_die:
                return list[index].m_fish_die;
                break;
            case audioType.Vox_f_fish:
                return list[index].f_fish;
                break;
            case audioType.Vox_f_fish_die:
                return list[index].f_fish_die;
                break;
            case audioType.Vox_m_boss:
                return list[index].m_boss;
                break;
            case audioType.Vox_m_boss_die:
                return list[index].m_boss_die;
                break;
            case audioType.Vox_f_boss:
                return list[index].f_boss;
                break;
            case audioType.Vox_f_boss_die:
                return list[index].f_boss_die;
                break;
        }

        return -1;
    },

    onDead () {
        if(this.type >= enemyType.boss_1){
            G.game.hitBoss++;
        }
        else{
            G.game.hitFish++;
        }

        //cc.log("onDead, " + G.game.fishCount);
        this.node.destroy();
    },
});
